* {
    font-family: "Dosis", sans-serif;
    font-size: 14px;
    color: #333333;
}

QObject {
    color: #1d1d1f;
    background-color: #f5f5f7;
}

QGroupBox {
    border: 1px solid #bdbdbd;
    border-radius: 4px;
    margin-top: 10px;
    font-weight: bold;
    background-color: #f5f5f5;
}

QStatusBar {
    color: #1d1d1f;
}

QMainWindow {
    background-color: #e0e0e0;
}

QTabWidget::pane {
    border: 1px solid #f5f5f7;
    background: #eeeeee;
    padding: 5px;
}

QTabWidget::pane::selection {
    border-top: 1px solid #333333;
}

QTabBar::tab {
    border: 1px solid #1d1d1f;
    padding: 5px;
    padding-right: 15px;
    padding-left: 15px;
    color: #1d1d1f;
    border-top-left-radius: 20px;
    border-top-right-radius: 5px;
    background: #f5f5f7;
    margin-bottom: -5px;
    font-weight: bolder;
    font-size: 16px;
}

QTabBar::tab:selected,
QTabBar::tab:hover {
    background: #424242;
    color: #f5f5f7;
    border-color: #1d1d1f;
}

QPushButton {
    border-radius: 10px;
    color: #1d1d1f;
    border: 2px solid #67bcc0;
    background-color: #dbeef0;
    color: #1d1d1f;
    padding: 5px;
}

QPushButton:disabled {
    background-color: #eeeeee;
    border: 2px solid #9e9e9e;
    color: #333333;
}

QPushButton:hover {
    border: 2px solid #1d1d1f;
    background-color: #67bcc0;
}

QPushButton:pressed {
    background-color: #53d4dc;
}

QPushButton:pressed {
    background-color: #53d4dc;
}

QPushButton#start_recording_button {
    font-size: 16px;
    background-color: #ce7076;
    border: 2px solid #333333;
    padding: 10px;
    border-radius: 10px;
    width: 250px;
}

QPushButton#start_recording_button:hover {
    border: 2px solid #cb2c34;
}

QPushButton#start_recording_button:disabled {
    background-color: #8c7474;
    color: #555555;
}

QPushButton#stop_recording_button {
    font-size: 14px;
    background-color: #eeeeee;
}

QPushButton#stop_recording_button:enabled {
    border: 3px solid #d95158;
    background-color: #eeeeee;
}

QPushButton#stop_recording_button:hover {
    border: 3px solid #1d1d1f;
    background-color: #eeeeee;
}

QPushButton#stop_recording_button:disabled {
    color: #555555;
}

QPushButton#process_motion_capture_button,
QPushButton#export_to_blender_button {
    font-size: 18px;
    background-color: #67bcc0;
    border: 2px solid #333333;
}

QPushButton#process_motion_capture_button:hover,
QPushButton#export_to_blender_button:hover {
    border: 2px solid #398588;
}

QPushButton#continue_unchanged_button {
    background-color: #dbeef0f9;
    border: 2px solid #67bcc0;
    color: #333333;
}

QPushButton#continue_unchanged_button:hover {
    border: 2px solid #1d1d1f;
    background-color: #67bcc0;
}

WelcomeScreenButton {
    border: 2px solid #333333;
    background-color: #67bcc0;
    color: #1d1d1f;
    font-size: 24px;
    border-radius: 20px;
    padding: 10px;
    min-height: auto;
}

WelcomeScreenButton:hover {
    border: 2px solid #398588;
}

WelcomeScreenButton[recommended_next=true] {
    background-color: #d95158;
    color: #f5f5f7;
}

WelcomeScreenButton[recommended_next=true]:hover {
    border: 2px solid #a1232a;
}

QLabel {
    font-size: 14px;
    color: #333333;
}

QTreeView QLabel {
    color: #1d1d1f;
}

QLabel:disabled {
    color: #e0e0e0;
}

QLineEdit {
    border: 1px solid #333333;
    border-radius: 4px;
    padding: 0 8px;
    color: #333333;
    background-color: #f5f5f7;
}

QLineEdit:disabled {
    border: 1px solid #bdbdbd;
    color: #333333;
    background-color: #e0e0e0;
}

QTreeView {
    background-color: #eeeeee;
    color: #1d1d1f;
    font-family: "Roboto", sans-serif;
    font-size: 12px;
}

QTreeView::item:has-children {
    background-color: #e0e0e0;
    color: #1d1d1f;
    border-bottom: 1px solid #bdbdbd;
}

QTreeView::item:selected,
QTreeView::item:hover {
    background-color: #f6e0f5;
    color: #333333;
}

LogViewWidget {
    background-color: #263238;
    color: #f5f5f7;
    font-family: "Roboto", sans-serif;
    font-size: 12px;
    padding: 8px;
}

QDialog {
    background-color: #f5f5f5;
}

QListWidget {
    background-color: #f5f5f5;
}
